/*
 *	On-screen Display
 *
 *	(c) 2020--2021 Jiri Kalvoda <jirikalvoda@kam.mff.cuni.cz>
 */

#include "osdd-exec.h"
#include "util.h"
#include "osdd-to-string.h"

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#include <unistd.h>



// *************************  STATE STRUCT **********

struct exec_state
{
	//struct osd_to_string_state to_string;
	bool do_not_insert_msg_to_que;
	bool no_fork;
	bool arg_by_line;
	VECTOR(char *) prefix;
	struct osd_to_string_state to_string;
};

// *************************  MAIN FUNCTIONS *******

static
void exec_write(struct exec_state *state, VECTOR(VECTOR(char)) in)
{
	if(!in) return;
	if(fork()==0)
	{
		VECTOR(char *) a = VCLONE(state->prefix);
		for(int i=0;i<VSIZE(in); i++) VPB(a,in[i]);
		VPB(a, 0);
		execvp(a[0], a);
	}
	for(int i=0;i<VSIZE(in); i++) VFREE(in[i]);
	VFREE(in);
}

CREATE_FUNC_USING_TO_STRINGS_VECTOR(exec)

// *************************  NEW ******************

static
void exec_new_help(FILE * f)
{
	fprintf(f,"\
Module EXEC help:\n\
\t-opt [ARG0] [ARG1] ...\n\
On each message execute\n\
ARG0 ARG1 ... [TIME if -t is set] LINE0 LINE1 ...\n\
" OSDD_TO_STRINGS_VECTOR_HELP "\
\n\
");
}

static 
struct osd_abstract exec_new(int argc, char ** argv, Display * nope)
{
	(void)nope;
	argc++;argv--;
	struct osd_abstract r;
	static const char short_opts[] = "+" OSDD_TO_STRINGS_VECTOR_SHORTOP;
	static const struct option long_opts[] = {
		OSDD_TO_STRINGS_VECTOR_LONGOP
		{ NULL,		0,			NULL,	0   },
	};
	memset(&r,0,sizeof(r));
	struct exec_state *state = xmalloc(sizeof(*state));
	memset(state,0,sizeof(state)[0]);

	osd_to_string_state_init(&state->to_string);

	int opt;
	optind = 0;
	while ((opt = getopt_long(argc, argv, short_opts, long_opts, NULL)) >= 0)
	{
		if(!osd_to_strings_vector_parse_arg(&state->to_string,opt))
		switch (opt)
		{
			default:
				fprintf(stderr,"Option %c not exist\n\n",opt);
				exec_new_help(stderr);
				exit(0);
		}
	}
	for(int ind = optind; ind < argc; ind++)
	{
		VPB(state->prefix, argv[ind]);
	}

	ADD_FUNC_TO_ABSTRACT(r, exec)
	return r;
}

// *************************  CREATOR ****************************

CREATE_STANDARD_CREATOR(exec , "EXEC")
