/*
 *	On-screen Display
 *
 *	(c) 2020--2021 Jiri Kalvoda <jirikalvoda@kam.mff.cuni.cz>
 */

#include "osdd-system.h"
#include "util.h"
#include "osdd-to-string.h"

#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <stdlib.h>
#include <unistd.h>



// *************************  STATE STRUCT **********

struct system_state
{
	struct osd_to_string_state to_string;
	bool no_fork;
	char * prefix;
};

// *************************  MAIN FUNCTIONS *******

static
void system_write(struct system_state *state, VECTOR(char) in)
{
	if(!in) return;
	if(state->no_fork || fork()==0)
	{
		if(state->prefix)
		{
			VECTOR(char) out = 0;
			vprint(&out,"%s%s",state->prefix,in);
			system(out);
			VFREE(out);
		}
		else
			system(in);
		if(!state->no_fork) _exit(0);
	}
	VFREE(in);
}

CREATE_FUNC_USING_TO_STRING(system)


// *************************  NEW ******************

static
void system_new_help(FILE * f)
{
	fprintf(f,"\
Module EXEC help:\n\
\t-opt [PREFIX]\n\
concatenatin of PREFIX and message will be executed by system(3)\n\
-F, --no-fork\t\tDo not fork before call system\n\
" OSDD_TO_STRING_HELP "\
\n\
");
}

static 
struct osd_abstract system_new(int argc, char ** argv, Display * nope)
{
	(void)nope;
	argc++;argv--;
	struct osd_abstract r;
	static const char short_opts[] = "+F" OSDD_TO_STRING_SHORTOP;
	static const struct option long_opts[] = {
		{ "no-fork",		no_argument,	NULL,	'F' },
		OSDD_TO_STRING_LONGOP
		{ NULL,		0,			NULL,	0   },
	};
	memset(&r,0,sizeof(r));
	struct system_state *state = xmalloc(sizeof(*state));
	memset(state,0,sizeof(state)[0]);

	osd_to_string_state_init(&state->to_string);

	int opt;
	optind = 0;
	while ((opt = getopt_long(argc, argv, short_opts, long_opts, NULL)) >= 0)
	{
		if(!osd_to_string_parse_arg(&state->to_string,opt))
		switch (opt)
		{
			case 'F':
				state->no_fork = 1;
				break;
			default:
				fprintf(stderr,"Option %c not exist\n\n",opt);
				system_new_help(stderr);
				exit(0);
		}
	}
	int ind = optind;
	if(ind < argc)
		state->prefix = argv[ind++];
	if(ind != argc)
	{
		fprintf(stderr,"Too many arguments\n\n");
		system_new_help(stderr);
		exit(0);

	}

	ADD_FUNC_TO_ABSTRACT(r, system)
	return r;
}

// *************************  CREATOR ****************************

CREATE_STANDARD_CREATOR(system,"SYSTEM")
